<?php
/**
 * Plugin Name: Give - Paytm Gateway
 * Plugin URI: https://github.com/impress-org/give-paytm
 * Description: Process online donations via the Paytm payment gateway.
 * Author: GiveWP
 * Author URI: https://givewp.com
 * Version: 1.1.1
 * Text Domain: give-paytm
 * Domain Path: /languages
 * GitHub Plugin URI: https://github.com/impress-org/give-paytm
 */

require_once 'vendor/autoload.php';
require_once 'vendor/vendor-prefixed/autoload.php';

if ( ! class_exists( 'Give_Paytm' ) ) {
	final class Give_Paytm {
		/**
		 * Instance.
		 *
		 * @since  1.0
		 * @access static
		 * @var
		 */
		static private $instance;

		/**
		 * Notices (array)
		 *
		 * @since 1.0.1
		 *
		 * @var array
		 */
		public $notices = array();

		/**
		 * Singleton pattern.
		 *
		 * @since  1.0
		 * @access private
		 */
		private function __construct() {
		}


		/**
		 * Get instance.
		 *
		 * @since  1.0
		 * @access static
		 * @return static
		 */
		static function get_instance() {
			if ( null === static::$instance ) {
				self::$instance = new self();
				self::$instance->setup();
			}

			return self::$instance;
		}


		/**
		 * Setup Give PayTM.
		 *
		 * @since  1.0.1
		 * @access private
		 */
		private function setup() {

			// Setup constants.
			$this->setup_constants();

			// Give init hook.
			add_action( 'give_init', array( $this, 'init' ), 10 );
			add_action( 'admin_init', array( $this, 'check_environment' ), 999 );
			add_action( 'admin_notices', array( $this, 'admin_notices' ), 15 );
		}

		/**
		 * Setup constants.
		 *
		 * @since  1.0
		 * @access public
		 */
		public function setup_constants() {
			// Global Params.
			define( 'GIVE_PAYTM_VERSION', '1.1.1' );
			define( 'GIVE_PAYTM_MIN_GIVE_VER', '3.3.1' );
			define( 'GIVE_PAYTM_BASENAME', plugin_basename( __FILE__ ) );
			define( 'GIVE_PAYTM_URL', plugins_url( '/', __FILE__ ) );
			define( 'GIVE_PAYTM_DIR', plugin_dir_path( __FILE__ ) );

			return self::$instance;
		}

		/**
		 * Load the text domain.
		 *
		 * @access private
		 * @since  1.0
		 *
		 * @return void
		 */
		public function load_textdomain() {

			// Set filter for plugin's languages directory.
			$give_paytm_lang_dir = dirname( plugin_basename( __FILE__ ) ) . '/languages/';
			$give_paytm_lang_dir = apply_filters( 'give_paytm_languages_directory', $give_paytm_lang_dir );

			// Traditional WordPress plugin locale filter
			$locale = apply_filters( 'plugin_locale', get_locale(), 'give-paytm' );
			$mofile = sprintf( '%1$s-%2$s.mo', 'give-paytm', $locale );

			// Setup paths to current locale file
			$mofile_local  = $give_paytm_lang_dir . $mofile;
			$mofile_global = WP_LANG_DIR . '/give-paytm/' . $mofile;

			if ( file_exists( $mofile_global ) ) {
				// Look in global /wp-content/languages/give-paytm folder
				load_textdomain( 'give-paytm', $mofile_global );
			} elseif ( file_exists( $mofile_local ) ) {
				// Look in local /wp-content/plugins/give-paytm/languages/ folder
				load_textdomain( 'give-paytm', $mofile_local );
			} else {
				// Load the default language files
				load_plugin_textdomain( 'give-paytm', false, $give_paytm_lang_dir );
			}

		}

		/**
		 * Set hooks
		 *
		 * @since  1.0.1
		 * @access public
		 */
		public function init() {

			if ( ! $this->get_environment_warning() ) {
				return;
			}

			$this->load_textdomain();
			$this->activation_banner();
			$this->licensing();

			if ( is_admin() ) {
				require_once GIVE_PAYTM_DIR . 'includes/admin/plugin-activation.php';
			}

			require_once GIVE_PAYTM_DIR . 'includes/functions.php';

			require_once GIVE_PAYTM_DIR . 'includes/admin/class-admin-settings.php';
			require_once GIVE_PAYTM_DIR . 'includes/payment-processing.php';
			require_once GIVE_PAYTM_DIR . 'includes/actions.php';
			require_once GIVE_PAYTM_DIR . 'includes/filters.php';
		}

		/**
		 * Implement Give Licensing for Give PayTM Add On.
		 *
		 * @since  1.0.1
		 * @access private
		 */
		private function licensing() {
			if ( class_exists( 'Give_License' ) ) {
				new Give_License( __FILE__, 'Paytm Gateway', GIVE_PAYTM_VERSION, 'WordImpress' );
			}
		}

		/**
		 * Check plugin environment.
		 *
		 * @since  1.0.1
		 * @access public
		 *
		 * @return bool
		 */
		public function check_environment() {
			// Flag to check whether plugin file is loaded or not.
			$is_working = true;

			// Load plugin helper functions.
			if ( ! function_exists( 'is_plugin_active' ) ) {
				require_once ABSPATH . '/wp-admin/includes/plugin.php';
			}

			/*
			 Check to see if Give is activated, if it isn't deactivate and show a banner. */
			// Check for if give plugin activate or not.
			$is_give_active = defined( 'GIVE_PLUGIN_BASENAME' ) ? is_plugin_active( GIVE_PLUGIN_BASENAME ) : false;

			if ( empty( $is_give_active ) ) {
				// Show admin notice.
				$this->add_admin_notice( 'prompt_give_activate', 'error', sprintf( __( '<strong>Activation Error:</strong> You must have the <a href="%s" target="_blank">Give</a> plugin installed and activated for Give - Paytm to activate.', 'give-paytm' ), 'https://givewp.com' ) );
				$is_working = false;
			}

			return $is_working;
		}

		/**
		 * Check plugin for Give environment.
		 *
		 * @since  1.0.1
		 * @access public
		 *
		 * @return bool
		 */
		public function get_environment_warning() {
			// Flag to check whether plugin file is loaded or not.
			$is_working = true;

			// Verify dependency cases.
			if (
				defined( 'GIVE_VERSION' )
				&& version_compare( GIVE_VERSION, GIVE_PAYTM_MIN_GIVE_VER, '<' )
			) {

				/*
				 Min. Give. plugin version. */
				// Show admin notice.
				$this->add_admin_notice( 'prompt_give_incompatible', 'error', sprintf( __( '<strong>Activation Error:</strong> You must have the <a href="%1$s" target="_blank">Give</a> core version %2$s for the Give - Paytm add-on to activate.', 'give-paytm' ), 'https://givewp.com', GIVE_PAYTM_MIN_GIVE_VER ) );

				$is_working = false;
			}

			return $is_working;
		}

		/**
		 * Allow this class and other classes to add notices.
		 *
		 * @since  1.0.1
		 *
		 * @param $slug
		 * @param $class
		 * @param $message
		 */
		public function add_admin_notice( $slug, $class, $message ) {
			$this->notices[ $slug ] = array(
				'class'   => $class,
				'message' => $message,
			);
		}

		/**
		 * Display admin notices.
		 *
		 * @since  1.0.1
		 */
		public function admin_notices() {

			$allowed_tags = array(
				'a'      => array(
					'href'  => array(),
					'title' => array(),
					'class' => array(),
					'id'    => array(),
				),
				'br'     => array(),
				'em'     => array(),
				'span'   => array(
					'class' => array(),
				),
				'strong' => array(),
			);

			foreach ( (array) $this->notices as $notice_key => $notice ) {
				echo "<div class='" . esc_attr( $notice['class'] ) . "'><p>";
				echo wp_kses( $notice['message'], $allowed_tags );
				echo '</p></div>';
			}

		}

		/**
		 * Show activation banner for this add-on.
		 *
		 * @since  1.0.1
		 *
		 * @return bool
		 */
		public function activation_banner() {

			// Check for activation banner inclusion.
			if (
				! class_exists( 'Give_Addon_Activation_Banner' )
				&& file_exists( GIVE_PLUGIN_DIR . 'includes/admin/class-addon-activation-banner.php' )
			) {
				include GIVE_PLUGIN_DIR . 'includes/admin/class-addon-activation-banner.php';
			}

			// Initialize activation welcome banner.
			if ( class_exists( 'Give_Addon_Activation_Banner' ) ) {

				// Only runs on admin.
				$args = array(
					'file'              => __FILE__,
					'name'              => esc_html__( 'Paytm Gateway', 'give-paytm' ),
					'version'           => GIVE_PAYTM_VERSION,
					'settings_url'      => admin_url( 'edit.php?post_type=give_forms&page=give-settings&tab=gateways&section=paytm' ),
					'documentation_url' => 'http://docs.givewp.com/addon-paytm',
					'support_url'       => 'https://givewp.com/support/',
					'testing'           => false, // Never leave true.
				);
				new Give_Addon_Activation_Banner( $args );
			}

			return true;
		}
	}

	/**
	 * Returns class object instance.
	 *
	 * @since 1.0.1
	 *
	 * @return Give_Paytm bool|object
	 */
	function Give_Paytm() {
		return Give_Paytm::get_instance();
	}

	Give_Paytm();
}

/**
 * Load the Service Providers with Give core.
 *
 * @since 1.1.1
 */
add_action('before_give_init', function () {
	// Check Give min required version.
	if (Give_Paytm()->check_environment()) {
		give()->registerServiceProvider(\GivePaytm\Paytm\ServiceProvider::class);
	}
});
