<?php

use Give\Donations\Models\Donation;
use Give\Helpers\Form\Utils;
use GivePaytm\Paytm\API\Models\Transaction;
use GivePaytm\Paytm\API\TransactionAPI;
use GivePaytm\Paytm\API\Translations;
use GivePaytm\Paytm\Models\Merchant;
use GivePaytm\Paytm\PaytmPaymentGateway;
use GivePaytm\Vendors\paytm\paytmchecksum\PaytmChecksum;

/**
 * Listen paytm ipn
 *
 * @since  1.0
 */
function give_paytm_listen_ipn() {
	if ( isset( $_GET['give-listener'] ) && $_GET['give-listener'] === 'PAYTM_IPN' ) {
		do_action( 'give_verify_paytm_gateway_ipn' );
	}
}

add_action( 'init', 'give_paytm_listen_ipn' );

/**
 * Process paytm ipn
 *
 * @since 1.1.1 Add support for legacy form, and v2 donation forms.
 * @since  1.0
 * @throws Exception
 */
function give_paytm_process_ipn() {
	$posted_data             = give_clean( $_POST );
	$get_data                = give_clean( $_GET );
	$is_valid_paytm_redirect = isset(
		$posted_data['ORDERID'],
		$posted_data['RESPCODE'],
		$posted_data['CHECKSUMHASH'],
		$get_data['payment-id'],
		$get_data['form-id']
	);

	// We should handle valid Paytm redirect.
	if ( ! $is_valid_paytm_redirect ) {
		$requiredPaytmEvents = ['APP_CLOSED', 'SESSION_EXPIRED'];
		$donation_id = isset( $get_data['payment-id'] ) ? absint( $get_data['payment-id'] ) : 0;
		$paytm_event_name = isset( $get_data['paytm-event-name'] )
			? sanitize_text_field( $get_data['paytm-event-name'] )
			: '';
		$donation    = Donation::find( $donation_id );

		// In case donor decline payment by closing paytm payment modal or session expired, we need to redirect to failed page.
		// Throw exception otherwise.
		if(
			$donation
			&& $donation->status->isPending()
			&& PaytmPaymentGateway::id() === $donation->gatewayId
			&& in_array( $paytm_event_name, $requiredPaytmEvents, true )
		) {
			$transaction = ( new TransactionAPI() )->getTransaction( $donation );

			if( $transaction->txnId && $transaction->isTransactionPending() ) {
				give_set_payment_transaction_id( $donation_id, $transaction->txnId );
				give_update_payment_status( $donation->id, 'failed' );
				give_insert_payment_note(
					$donation->id,
					sprintf(
						__( 'Transaction Failed. Paytm response message:  %s', 'give-paytm' ),
						$paytm_event_name
					)
				);

				if ( Utils::isLegacyForm( $donation->formId ) ) {
					wp_redirect( sprintf(
						'%1$s?give-paytm-payment=failed&give-paytm-error-message=%2$s',
						give_get_failed_transaction_uri(),
						base64_encode( esc_html__( 'Donor decline payment.', 'give-paytm' ) )
					) );
					exit();
				}

				// Redirect to failed page. In case of new form, we need to pass query string to show success page in iframe.
				give_send_to_success_page( '?giveDonationFormInIframe=1' );
			}
		}

		throw new RuntimeException( 'Invalid redirect from Paytm.' );
	}

	$merchant          = give( Merchant::class );
	$is_valid_checksum = PaytmChecksum::verifySignature(
		$posted_data,
		$merchant->accessKey,
		$posted_data['CHECKSUMHASH']
	);

	// We should handle valid Paytm checksum.
	if ( ! $is_valid_checksum ) {
		throw new RuntimeException( 'Invalid checksum from Paytm.' );
	}

	$donation_id = isset( $get_data['payment-id'] ) ? absint( $get_data['payment-id'] ) : 0;
	$form_id     = isset( $get_data['form-id'] ) ? absint( $get_data['form-id'] ) : 0;
	$donation    = Donation::find( $donation_id );

	// Donation should exist in database.
	if ( ! $donation || $donation->formId !== $form_id ) {
		throw new RuntimeException( 'Invalid donation.' );
	}

	if ( ! $donation->status->isPending() ) {
		throw new RuntimeException( 'Donation is not pending.' );
	}

	// Donation should be process with Paytm.
	if ( PaytmPaymentGateway::id() !== $donation->gatewayId ) {
		throw new RuntimeException( 'Invalid payment gateway.' );
	}

	$transaction = Transaction::makePaytmPostedData( $posted_data );

	// Paytm transaction data should match with donation.
	if ( $donation->id !== $transaction->orderId ) {
		throw new RuntimeException( 'Paytm transaction order id does not match with donation.' );
	}

	if ( $transaction->isTransactionSuccessful() ) {
		$amountFromPreviousTransaction = $transaction->txnAmount;
		// Refresh transaction to get latest transaction data.
		$transaction = ( new TransactionAPI() )->getTransaction( $donation );

		// Re-confirm transaction.
		if (
			$donation->id === $transaction->orderId
			&& $amountFromPreviousTransaction === $transaction->txnAmount
			&& $transaction->isTransactionSuccessful()
		) {
			give_insert_payment_note(
				$donation_id,
				sprintf( __( 'Transaction Successful. Paytm Transaction ID: %s', 'give-paytm' ),
					$transaction->txnId
				)
			);
			give_set_payment_transaction_id( $donation_id, $transaction->txnId );
			update_post_meta( $donation_id, 'paytm_donation_response', $posted_data );
			give_update_payment_status( $donation_id, 'complete' );

			// Redirect to success page. In case of legacy form, we need to pass empty string.
			// In case of new form, we need to pass query string to show success page in iframe.
			$queryString = Utils::isLegacyForm($form_id) ? '' : '?giveDonationFormInIframe=1';
			give_send_to_success_page($queryString);
		}
	}

	$response_msg = esc_attr( give( Translations::class )
		->getTranslation(
			'transactionStatus',
			$posted_data['RESPCODE'] ) ?? $posted_data['RESPMSG']
	);

	give_record_gateway_error(
		__( 'Paytm Error', 'give-paytm' ),
		sprintf( __( 'Transaction Failed. Paytm response message: %s', 'give-paytm' ),
			$response_msg ) . '<br><br>' . sprintf( esc_attr__( 'Details: %s', 'give-paytm' ),
			'<br>' . print_r( $_REQUEST, true ) ),
		$donation->id
	);

	give_update_payment_status( $donation->id, 'failed' );
	update_post_meta( $donation->id, 'paytm_donation_response', $posted_data );
	give_insert_payment_note(
		$donation->id,
		sprintf(
			__( 'Transaction Failed.Paytm response message:  %s', 'give-paytm' ),
			$response_msg
		)
	);

	if( Utils::isLegacyForm($form_id) ) {
		wp_redirect( give_get_failed_transaction_uri() . '?give-paytm-payment=failed&give-paytm-error-message=' . base64_encode( $response_msg )  );
		exit();
	}

	// Redirect to failed page. In case of new form, we need to pass query string to show success page in iframe.
	give_send_to_success_page( '?giveDonationFormInIframe=1' );
}

add_action( 'give_verify_paytm_gateway_ipn', 'give_paytm_process_ipn' );

