<?php

namespace GivePaytm\Paytm\API;

use Give\Donations\Models\Donation;
use GivePaytm\Paytm\API\Exceptions\CreateTransactionAPIException;
use GivePaytm\Paytm\API\Models\Transaction;
use RuntimeException;

/**
 * Class TransactionAPI
 * @since 1.1.1
 */
class TransactionAPI extends Request {
	/**
	 * This function should be used to create transaction.
	 *
	 * Paytm rest api - https://business.paytm.com/docs/api/initiate-transaction-api
	 *
	 * @since 1.1.1
	 * @throws CreateTransactionAPIException|RuntimeException
	 */
	public function createTransaction( Donation $donation ): string {
		$requestUrl = sprintf(
			'/theia/api/v1/initiateTransaction?mid=%1$s&orderId=%2$s',
			$this->merchant->merchantId,
			$donation->id
		);

		$requestArgs = $this->buildParamsForCreateTransaction( $donation );
		$response    = $this->execute( $requestUrl, $requestArgs );

		$this->validateCreateTransactionResponse( $response );

		return $response->body->txnToken;
	}

	/**
	 * This function should be used to get transaction.
	 * @since 1.1.1
	 */
	public function getTransaction( Donation $donation ): Transaction {
		$requestUrl = '/v3/order/status';

		$requestArgs = [
			'mid'     => $this->merchant->merchantId,
			'orderId' => $donation->id,
		];

		$response = $this->execute( $requestUrl, $requestArgs );

		return Transaction::makeObject( $response );
	}

	/**
	 * This function return request body for create transaction rest api request.
	 *
	 * @since 1.1.1
	 */
	private function buildParamsForCreateTransaction( Donation $donation ): array {
		$callbackURL = sprintf(
			'%1$s/?give-listener=PAYTM_IPN&payment-id=%2$s&form-id=%3$s',
			get_site_url(),
			$donation->id,
			$donation->formId
		);

		return [
			'requestType' => 'Payment',
			'mid'         => $this->merchant->merchantId,
			'websiteName' => $this->merchant->websiteName,
			'orderId'     => $donation->id,
			'callbackUrl' => $callbackURL,
			'txnAmount'   => [
				'value'    => $donation->amount->formatToDecimal(),
				'currency' => $donation->amount->getCurrency(),
			],
			'userInfo'    => [
				'custId' => $donation->email,
			]
		];
	}

	/**
	 * This function should be used to validate create transaction response.
	 *
	 * @since 1.1.1
	 * @throws CreateTransactionAPIException
	 */
	private function validateCreateTransactionResponse( object $response ): void {
		$resultInfo = $response->body->resultInfo;

		if ( 'S' !== $resultInfo->resultStatus ) {
			throw new CreateTransactionAPIException(
				$resultInfo->resultStatus,
				$resultInfo->resultCode,
				$resultInfo->resultMsg
			);
		}
	}
}
