<?php

namespace GivePaytm\Paytm\Controllers;

use Give\Donations\Models\Donation;
use GivePaytm\Paytm\Models\Merchant;
use GivePaytm\Paytm\PaytmPaymentGateway;

/**
 * This class is used display Paytm payment modal on receipt aka success page after successful pending donation.
 *
 * @since 1.1.1
 */
class HandleProcessPaymentRedirect {
	/**
	 * @since 1.1.1
	 */
	public function __invoke(): void {
		$donationSessionData = Give()->session->get( 'give_purchase' );
		$donationId          = $donationSessionData['donation_id'];
		$donation            = Donation::find( $donationId );

		if ( ! $donation ) {
			return;
		}

		// Only handle redirect for donation with paytm gateway.
		if ( $donation->gatewayId !== PaytmPaymentGateway::id() ) {
			return;
		}

		// Only handle redirect for pending donation.
		if ( ! $donation->status->isPending() ) {
			return;
		}

		$merchant    = give( Merchant::class );
		$txnToken    = give()->session->get( 'paytm_txn_token' );
		$redirectUrl = add_query_arg(
			[
				'give-listener'    => 'PAYTM_IPN',
				'payment-id'       => $donation->id,
				'paytm-event-name' => 'paytmEventName'
			],
			home_url()
		);

		$paytmDonationProcessCommunicationMessage = apply_filters(
			'give_paytm_donation_process_communication_message',
			esc_html__( 'Please proceed with the Paytm transaction when the payment window appears. You\'ll be redirected to a receipt page upon completion.', 'give-paytm' )
		);

		ob_start();
		?>
		<script>
			function onScriptLoad() {
				if ( window.Paytm && window.Paytm.CheckoutJS ) {
					const config = {
						"root": "",
						"flow": "DEFAULT",
						"data": {
							"orderId": "<?php echo esc_js( $donation->id ); ?>",
							"token": "<?php echo esc_js( $txnToken ); ?>",
							"tokenType": "TXN_TOKEN"
						},
						"handler": {
							"notifyMerchant": function ( eventName, data ) {
								console.log( "notifyMerchant handler function called" );
								console.log( "eventName => ", eventName );
								console.log( "data => ", data );

								// Redirect to failed page.
								switch ( eventName ) {
									case 'APP_CLOSED':
									case 'SESSION_EXPIRED':
										window.location.href = '<?php echo esc_url_raw( $redirectUrl ); ?>'.replace( 'paytmEventName', eventName );
										console.log( 'Refresh donation status and redirect to failed or confirmation page.' );
										break;
								}
							}
						}
					};

					window.Paytm.CheckoutJS.onLoad( function executeAfterCompleteLoad() {
						const removeWaitingMessage = function(){
							// Remove waiting message container.
							const waitingMessageContainer = document.getElementById( 'give-paytm-waiting-message' );
							waitingMessageContainer && waitingMessageContainer.remove();
						}

						// initialze configuration using init method
						window.Paytm.CheckoutJS.init( config ).then( function onSuccess() {
							// after successfully updating configuration, invoke JS Checkout
							window.Paytm.CheckoutJS.invoke();
							removeWaitingMessage();
						} ).catch( function onError( error ) {
							console.log( "error => ", error );
						} ).finally( function onFinish() {
							removeWaitingMessage()
						} );
					} );
				}
			}

			window.addEventListener( 'load', function() {
				const waitingMessageContainer = document.createElement( 'div' );
				const waitingMessage = document.createElement( 'p' );

				waitingMessageContainer.id = 'give-paytm-waiting-message';

				// Add style to message container.
				waitingMessageContainer.style.position = 'fixed';
				waitingMessageContainer.style.top = '0';
				waitingMessageContainer.style.right = '0';
				waitingMessageContainer.style.bottom = '0';
				waitingMessageContainer.style.left = '0';
				waitingMessageContainer.style.zIndex = '10000000000';
				waitingMessageContainer.style.background = 'rgba(0,0,0,.5)';
				waitingMessageContainer.style.display = 'flex';
				waitingMessageContainer.style.justifyContent = 'center';
				waitingMessageContainer.style.alignItems = 'center';

				// Add message & style.
				waitingMessage.style.fontSize = '20px';
				waitingMessage.style.color = '#fff';
				waitingMessage.innerHTML = '<?php echo esc_js( $paytmDonationProcessCommunicationMessage ); ?>';

				// Append message container to body.
				waitingMessageContainer.appendChild( waitingMessage );
				document.body.appendChild( waitingMessageContainer );
			} );
		</script>
		<script type="application/javascript"
				src="<?php echo esc_url( $merchant->getRequestEndpoint() ); ?>/merchantpgpui/checkoutjs/merchants/<?php echo $merchant->merchantId; ?>.js"
				onload="onScriptLoad();"
				crossorigin="anonymous"></script>
		<?php
		$markup = ob_get_clean();

		// Show paytm payment modal on legacy donation forms.
		add_filter( 'the_content', function ( $content ) use ( $markup ) {
			return $content . $markup;
		}, 99 );

		// Show paytm payment modal on v2 donation forms.
		add_action( 'give_embed_head', function () use ( $markup ) {
			echo $markup;
		} );
	}
}
